<?php
/**
 * @package Smartsub (Smart Subscriptions) for Joomla 3
 * @version 1.0
 * @author Alex Polonski
 * @copyright (C)2014 - Alex Polonski
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 **/
defined('_JEXEC') or die;

define('COM_SMARTSUB_PAYMENTS_INITUSERS_CHUNK_SIZE', 1000);

require_once JPATH_ADMINISTRATOR . '/components/com_smartsub/helpers/smartsub.php';

class SmartsubControllerPayments extends JControllerAdmin
{
	public function __construct($config = array())
	{
		parent::__construct($config);
		$this->text_prefix = 'COM_SMARTSUB_PAYMENTS';
		
		$this->registerTask('setpaid', 'setstatus');
		$this->registerTask('setpending', 'setstatus');
	}
	
	public function getModel($name = 'Payment', $prefix = 'SmartsubModel', $config = array('ignore_request' => true))
	{
		$model = parent::getModel($name, $prefix, $config);

		return $model;
	}

	public function setStatus()
	{
		// Check for request forgeries
		JSession::checkToken() or jexit(JText::_('JINVALID_TOKEN'));
	
		$pks	= $this->input->post->get('cid', array(), 'array');
		$values	= array('setpaid' => 'paid', 'setpending' => 'pending');
		$task	= $this->getTask();
		$status	= JArrayHelper::getValue($values, $task, 'pending', 'cmd');

		try
		{
			JArrayHelper::toInteger($pks);
			if (empty($pks))
			{
				throw new Exception(JText::_('COM_SMARTSUB_NO_PAYMENT_SELECTED'));
			}
			
			$model = $this->getModel();
			$result = $model->setPaid($pks, $status);
			$this->setMessage(JText::plural('COM_SMARTSUB_PAYMENTS_SUCCESS_N_STATUS_SET', (int)$result), !empty($result) ? 'message' : 'notice');
		}
		catch (Exception $e)
		{
			JError::raiseWarning(500, $e->getMessage());
		}
	
		$this->setRedirect('index.php?option=com_smartsub&view=payments');
	}
	
	public function initusers()
	{
		$user = JFactory::getUser();
		// Only super-admins allowed here
		if(!$user->authorise('core.admin'))
		{
			$this->setRedirect('index.php?option=com_smartsub&view=payments', JText::_('JERROR_ALERTNOAUTHOR'), 'warning');
			return false;
		}
		
		// If a free account is not found, do not proceed
		$free_account_id = SmartsubHelper::findFreeAccount();
		if(empty($free_account_id))
		{
			$this->setRedirect('index.php?option=com_smartsub&view=payments', JText::_('COM_SMARTSUB_PAYMENTS_INITUSERS_NO_FREE_ACCOUNT'), 'notice');
			return true;
		}
		
		// get free account data
		JTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_smartsub/tables');
		$account_table = JTable::getInstance('Account', 'SmartsubTable');
		if($account_table->load($free_account_id))
		{
			$free_group_id = $account_table->group_id;
			
			// Look for users who have the only usergroup assigned ($free_group_id) and do not have
			// "account" in their profiles
			$db = JFactory::getDbo();
			$query = $db->getQuery(true);
			
			$query->select('a.id')
				->from('#__users AS a')
				->leftJoin('#__user_usergroup_map AS b ON b.user_id = a.id AND b.group_id <> ' . (int)$free_group_id)
				->select('MAX(d.ordering) AS max_ordering')
				->leftJoin('#__user_profiles AS d ON d.user_id = a.id')
				->leftJoin('#__user_profiles AS c ON c.user_id = a.id AND c.profile_key = ' . $db->Quote('smartsub.account'))
				->where('b.user_id IS NULL')
				->where('c.profile_value IS NULL')
				->group('a.id');
			
			//$db->setQuery($query, 0, 1000);
			$db->setQuery($query);
			
			try
			{
				$users = $db->loadObjectList();
			}
			catch(RuntimeException $e)
			{
				$this->setRedirect('index.php?option=com_smartsub&view=payments', $e->getMessage(), 'warning');
				return false;
			}
			
			// If no qualified users found, do not proceed
			if(empty($users))
			{
				$this->setRedirect('index.php?option=com_smartsub&view=payments', JText::_('COM_SMARTSUB_PAYMENTS_INITUSERS_NO_USERS'), 'notice');
				return true;
			}
			
			// If there are thousands of users to process, MySQL can report max_packet_size error,
			// so we split the operation in chunks. We will add a message for admin.
			if(count($users) > COM_SMARTSUB_PAYMENTS_INITUSERS_CHUNK_SIZE)
			{
				$users = array_slice($users, 0, COM_SMARTSUB_PAYMENTS_INITUSERS_CHUNK_SIZE);
				$is_partial = true;
			}
			
			// Start assignment
			$db->transactionStart();
			
			// we have to modify both user_profiles and payments tables
			$tuples_profile = array();
			$tuples_payment = array();
			
			foreach($users as $user)
			{	
				$tuples_profile[] = '(' . $user->id . ', ' . $db->Quote('smartsub.account') . ', ' . $db->quote(json_encode($free_account_id)) . ', ' . ($user->max_ordering + 1) . ')';
				$tuples_payment[] = '(' . $user->id . ', ' 
						. $free_account_id . ', ' 
						. $db->Quote($db->getNullDate()) . ', ' 
						. $db->Quote(JDate::getInstance()->toSql()) 
						. ', 0, 0, 0, ' 
						. $db->Quote('EUR') . ', ' 
						. $db->Quote('none') . ', ' 
						. $db->Quote('profile') . ', 0, ' 
						. $db->Quote('paid')
				. ')';
			}
				
			try
			{
				$db->setQuery('INSERT INTO #__user_profiles VALUES ' . implode(', ', $tuples_profile));
				$db->execute();
				$db->setQuery('INSERT INTO #__ss_payments (user_id,account_id,expires,date,amount,discount,upgrade_from,currencyID,system,task,offline,status) VALUES ' . implode(', ', $tuples_payment));
				$db->execute();
			}
			catch(RuntimeException $e)
			{
				$db->transactionRollback();
				$this->setRedirect('index.php?option=com_smartsub&view=payments', $e->getMessage(), 'warning');
				return false;
			}
			
			// Success
			$db->transactionCommit();
			
			$message = JText::plural('COM_SMARTSUB_PAYMENTS_INITUSERS_N_PROCESSED', count($users));
			$type = 'message';
			if(!empty($is_partial))
			{
				// not all users were processed. Alter message and type
				$message .= '<br />' . JText::_('COM_SMARTSUB_PAYMENTS_INITUSERS_PARTIAL');
				$type = 'notice';
			}
			
			$this->setRedirect('index.php?option=com_smartsub&view=payments', $message, $type);
			return true;
		}
		else
		{
			// critical error - account data was not loaded
			$this->setRedirect('index.php?option=com_smartsub&view=payments', JText::_('JLIB_DATABASE_ERROR_EMPTY_ROW_RETURNED'), 'warning');
			return false;
		}
	}
	
	
	protected function postDeleteHook(JModelLegacy $model, $ids = null)
	{
	}
}
