<?php
/**
 * @package Smartsub (Smart Subscriptions) for Joomla 3
 * @version 1.0
 * @author Alex Polonski
 * @copyright (C)2014 - Alex Polonski
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 **/
defined('JPATH_PLATFORM') or die;

JHtml::_('behavior.framework', true);

/**
 * Utility class for Smartsub Custom JavaScript behaviors
 */
abstract class JHtmlSmartsubBehavior
{
	/**
	 * Array containing information for loaded files
	 *
	 * @var    array
	 * @since  2.5
	 */
	protected static $loaded = array();

	/**
	 * Method to load custom validation rules
	 *
	 */
	public static function customvalidation()
	{
		// Only load once
		if (!empty(static::$loaded[__METHOD__]))
		{
			return;
		}
		
		$params = JComponentHelper::getParams('com_smartsub');
		$decimal_point_char = JText::_('DECIMALS_SEPARATOR');
		$thousands_separator_char = JText::_('THOUSANDS_SEPARATOR');
		$currency_decimals = $params->get('currency_decimals', 2);

		// $$$ at the moment we do not allow negative values!
		$price_regex = $currency_decimals > 0 ? '/^\d+(' . $decimal_point_char . '\d{1,' . $currency_decimals .'})?$/' : '/^\d+$/';
		$percentage_regex = '/^\d+(' . $decimal_point_char . '\d{1,2})?$/';
				
		// Attach custom validation handlers used by JFormValidator
		$validation_script	= 	"window.addEvent('domready', function() {
									// percentage: no thousands separator allowed, max. 2 decimal points
									document.formvalidator.setHandler('smartsubpercentage', function(value) {
										var regex	= " . $percentage_regex . ";
										return regex.test(value);
									});
									// price: thousands separator allowed but not required, max. decimal points from settings
									document.formvalidator.setHandler('smartsubprice', function(value) {
										value = value.replace('" . $thousands_separator_char . "', '');
										var regex	= " . $price_regex . ";
										return regex.test(value);
									});
								});";
		JFactory::getDocument()->addScriptDeclaration($validation_script);
		
		// ** validate-whatever-other-validation-rules-we-may-want-to-implement :: New validation rules handlers go here
		
		// Set static array
		static::$loaded[__METHOD__] = true;

		return;
	}
	
	/**
	 * Method to load custom replace rules
	 *
	 */
	public static function customreplace()
	{
		// Only load once
		if (!empty(static::$loaded[__METHOD__]))
		{
			return;
		}
	
		$params = JComponentHelper::getParams('com_smartsub');
		$decimal_point_char = JText::_('DECIMALS_SEPARATOR');
		$thousands_separator_char = JText::_('THOUSANDS_SEPARATOR');
		$currency_decimals = $params->get('currency_decimals', 2);
		
		// Attach custom replace event handlers
	
		$validation_script	= 	"window.addEvent('domready', function(){
									// format numbers in math format before submitting data
									document.getElements('form.form-validate').addEvent('submit', function() {
										var val, inputs = document.getElements('input[class*=\"validate-smartsub\"]');
										inputs.each(function(el) {
											// here we can set the value back to standard floating point format
											val = el.get('value');
											val = val.replace('" . $thousands_separator_char . "', '');
											val = val.replace('" . $decimal_point_char . "', '.');
											el.set('value', val);
										});
									});
									// format numbers in local format before displaying on the form
									document.getElements('input[class*=\"validate-smartsub\"]').each(function(el) {
										var decimals = el.hasClass('validate-smartsubprice') ? " . $currency_decimals . " : 2;
										val = +el.get('value');
										// use mootools.more
										val = val.format({
											decimal : '" . $decimal_point_char . "',
											group : '" . $thousands_separator_char . "',
											decimals : decimals
										});		
										el.set('value', val);
									});
								});";
		JFactory::getDocument()->addScriptDeclaration($validation_script);
	
		// Set static array
		static::$loaded[__METHOD__] = true;
	
		return;
	}

}
