<?php
/**
 * @package Smartsub (Smart Subscriptions) for Joomla 3
 * @version 1.0
 * @author Alex Polonski
 * @copyright (C)2014 - Alex Polonski
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 **/
defined('_JEXEC') or die;

class SmartsubModelPayments extends JModelList
{
	public function __construct($config = array())
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = array(
				'id', 'a.id',
				'account_title', 'ac.title',
				'date', 'a.date',
				'system', 'a.system',
				'checked_out', 'a.checked_out',
				'checked_out_time', 'a.checked_out_time',
				'status', 'a.status', 'status_title',
				'amount', 'a.amount',
				'currencyID', 'a.currencyID',
				'expires', 'a.expires',
				'user_name', 'u.name', 'a.user_id', 'user_id',
				'a.task', 'task'
			);
		}

		parent::__construct($config);
	}

	protected function populateState($ordering = null, $direction = null)
	{
		// Load the parameters.
		$params = JComponentHelper::getParams('com_smartsub');
		$this->setState('params', $params);
		
		// List state information.
		parent::populateState('a.date', 'desc');
	}

	protected function getStoreId($id = '')
	{
		// Compile the store id.
		$id .= ':' . $this->getState('filter.search');
		$id .= ':' . $this->getState('filter.status');
		$id .= ':' . $this->getState('filter.date');
		$id .= ':' . $this->getState('filter.expires');
		$id .= ':' . $this->getState('filter.system');
		$id .= ':' . $this->getState('filter.task');
		$id .= ':' . $this->getState('filter.user_id');
		
		return parent::getStoreId($id);
	}

	/**
	 * Build an SQL query to load the list data.
	 *
	 * @return  JDatabaseQuery
	 * @since   1.6
	 */
	protected function getListQuery()
	{
		// Create a new query object
		$db = $this->getDbo();
		$query = $db->getQuery(true);
		$user = JFactory::getUser();

		// Select the required fields from the table.
		$query->select(
			$this->getState(
				'list.select',
				'a.id, a.account_id, a.date, a.system, a.task, a.checked_out, a.checked_out_time,' .
					'a.upgrade_from, a.amount, a.currencyID, a.discount, a.status, a.expires, a.transactionID'
			)
		);
		$query->from($db->quoteName('#__ss_payments') . ' AS a');
		
		// Join over the users for the checked out user
		$query->select('uc.name AS editor')
			->join('LEFT', '#__users AS uc ON uc.id=a.checked_out');

		// Join over pr_accounts
		$query->select('ac.title AS account_title')
			->join('LEFT', '#__ss_accounts AS ac ON ac.id = a.account_id');
		
		// Join over pr_accounts for upgrade_from account title
		$query->select('oac.title AS upgrade_from_account')
			->join('LEFT', '#__ss_accounts AS oac ON oac.id = a.upgrade_from');

		// Join over the users
		$query->select('u.name AS user_name, u.username AS user_login')
			->join('LEFT', '#__users AS u ON u.id = a.user_id');
		
		// Filter by status
		$status = $this->getState('filter.status');
		if (!empty($status))
		{
			// status filter can be a '|'-separated list of statuses
			$status = $db->Quote(explode('|', $status));
			$query->where('a.status IN (' . implode(',', $status) . ')');
		}
		
		// Filter by payment method
		$system = $this->getState('filter.system');
		if (!empty($system))
		{
			$query->where('a.system = ' . $db->Quote($system));
		}
		
		// Filter by task 
		$task = $this->getState('filter.task');
		if (!empty($task))
		{
			$query->where('a.task = ' . $db->Quote($task));
		}
		
		// Filter by payment date range
		$date = $this->getState('filter.date');
		if(!empty($date))
		{
			$dNow   = new JDate;
			$dStart = clone $dNow;
		
			switch ($date)
			{
				case 'past_week':
					$dStart->modify('-7 day');
					break;
		
				case 'past_1month':
					$dStart->modify('-1 month');
					break;
		
				case 'past_3month':
					$dStart->modify('-3 month');
					break;
		
				case 'post_6month':
				case 'past_6month':
					$dStart->modify('-6 month');
					break;
		
				case 'today':
					// Ranges that need to align with local 'days' need special treatment.
					$app    = JFactory::getApplication();
					$offset = $app->get('offset');
		
					// Reset the start time to be the beginning of today, local time.
					$dStart = new JDate('now', $offset);
					$dStart->setTime(0, 0, 0);
		
					// Now change the timezone back to UTC.
					$tz = new DateTimeZone('GMT');
					$dStart->setTimezone($tz);
					break;
			}
		
			if($date == 'post_6month')
			{
				$query->where(
						'a.date < ' . $db->quote($dStart->format('Y-m-d H:i:s'))
				);
			}
			else
			{
				$query->where(
						'a.date >= ' . $db->quote($dStart->format('Y-m-d H:i:s')) .
						' AND a.date <=' . $db->quote($dNow->format('Y-m-d H:i:s'))
				);
			}
		}
		
		// Filter by expires date range
		$date = $this->getState('filter.expires');
		if(!empty($date))
		{
			$dNow   = new JDate;
			$dStart = clone $dNow;
		
			switch ($date)
			{
				case 'next_1week':
					$dStart->modify('+7 day');
					break;
		
				case 'next_2week':
					$dStart->modify('+14 day');
					break;
		
				case 'next_1month':
				case 'next_1month_plus':
					$dStart->modify('+1 month');
					break;
		
				case 'today':
				case 'tomorrow':
					// Ranges that need to align with local 'days' need special treatment.
					$app    = JFactory::getApplication();
					$offset = $app->get('offset');
		
					// Reset the start time to be the beginning of today, local time.
					$dStart = new JDate('now', $offset);
					$dStart->setTime(0, 0, 0);
		
					// Now change the timezone back to UTC.
					$tz = new DateTimeZone('GMT');
					$dStart->setTimezone($tz);
					
					$dStart->modify('+1 day');
					break;
			}
			
			if($date == 'tomorrow')
			{
				// for tomorrow only select this day range
				$dNow = clone($dStart);
				$dStart->modify('+1 day');
			}
		
			if($date == 'never')
			{
				$query->where(
						'a.expires = ' . $db->quote($db->getNullDate())
				);
			}
			elseif($date == 'next_1month_plus')
			{
				$query->where(
						'a.expires > ' . $db->quote($dStart->format('Y-m-d H:i:s'))
				);
			}
			elseif($date == 'expired')
			{
				$query->where(
						'a.expires < ' . $db->quote($dNow->format('Y-m-d H:i:s')) .
						' AND a.expires <>' . $db->quote($db->getNullDate())
				);
			}
			else
			{
				$query->where(
						'a.expires <= ' . $db->quote($dStart->format('Y-m-d H:i:s')) .
						' AND a.expires >=' . $db->quote($dNow->format('Y-m-d H:i:s'))
				);
			}
		}
		
		// Filter by search in title
		$search = $this->getState('filter.search');
		if (!empty($search))
		{
			if (stripos($search, 'id:') === 0)
			{
				$query->where('a.id = ' . (int) substr($search, 3));
			}
			else
			{
				$search = $db->quote('%' . $db->escape($search, true) . '%');
				$query->where('(u.name LIKE ' . $search . ' OR u.username LIKE ' . $search . ' OR a.transactionID LIKE ' . $search . ' OR ac.title LIKE ' . $search . ')');
			}
		}
		
		// Filter by user id
		$user_id = $this->getState('filter.user_id');
		if (!empty($user_id))
		{
			$query->where('a.user_id = ' . (int)$user_id);
		}

		// Add the list ordering clause.
		$orderCol = $this->state->get('list.ordering');
		$orderDirn = $this->state->get('list.direction');
		$query->order($db->escape($orderCol . ' ' . $orderDirn));

		$query->group('a.id');
		
		//echo nl2br(str_replace('#__','jos_',$query));
		return $query;
	}
}
