<?php
/**
 * @package plg_content_smartsub (Smart Subscriptions Content ACL plugin - readmore access) for Joomla 3
 * @version 1.1
 * @author Alex Polonski
 * @copyright (C)2014 - Alex Polonski
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 **/
// No direct access.
defined('_JEXEC') or die;

class PlgContentSmartsub extends JPlugin
{
	/**
	 * Load the language file on instantiation.
	 *
	 * @var    boolean
	 * @since  3.1
	 */
	protected $autoloadLanguage = true;
	
	// Smartsub component presense flag
	protected $componentInstalled = false;
	
	public function __construct(& $subject, $config)
	{
		parent::__construct($subject, $config);
		$this->componentInstalled = JComponentHelper::isEnabled('com_smartsub') && JPluginHelper::isEnabled('user', 'smartsub');
	}

	public function onContentPrepare($context, &$row, &$params, $page = 0)
	{
		$app = JFactory::getApplication();
		$canProceed = $app->isSite() && $context == 'com_content.article' && $this->componentInstalled;

		// if there is no fulltext, there is nothing to do
		if (!$canProceed || empty($row->fulltext))
		{
			return;
		}
		
		// no action for users outside Smartsub scope
		if(!$this->_isSmartsubScope())
		{
			return;
		}
		
		$attribs = new JRegistry();
		$attribs->loadString($row->attribs);
		$fulltext_access = $attribs->get('fulltext_access', '');
		if(empty($fulltext_access))
		{
			return;
		}
		
		$input = $app->input;
		
		// We can detect that this view is for printing, so we will add
		// "content hidden" message instead of a link to subscription purchase
		$print = $input->getBool('print');
		$view = $input->getString('view');
		
		$user = JFactory::getUser();
		$levels = JAccess::getAuthorisedViewLevels($user->get('id', 0));

		if(!empty($row->fulltext) && !in_array($fulltext_access, $levels))
		{
			// If we get to this point current users is not allowed to view
			// full article text
			if(!empty($print) || $view != 'article')
			{
				$row->fulltext = JText::_('PLG_CONTENT_SMARTSUB_ACCESS_DENIED_PRINT');
			}
			else
			{
				// screen view (check if there are no other options, like feeds, etc...)
				
				// get subscriptions that offer required view level
				$accounts = $this->_getRecommendedAccounts($fulltext_access, $levels);
				if(empty($accounts))
				{
					$row->fulltext = JText::_('PLG_CONTENT_SMARTSUB_ACCESS_DENIED_NOACCOUNTS');
				}
				else 
				{
					// save recommended accounts in session, so that account selection will be limited
					// to these accounts in registration and profile forms
					// *** has to be cleaned up in user plugin after successfull save 
					$app->setUserState('com_smartsub.recommended_accounts', $accounts);
					if($user->guest)
					{
						// guests - redirect to registration
						$link = JRoute::_('index.php?option=com_users&view=registration&propose_acc=1', false);
					}
					else 
					{
						// logged in - redirect to profile
						$link = JRoute::_('index.php?option=com_users&view=profile&layout=edit&propose_acc=1&id=' . $user->get('id'), false);
					}
					
					$row->fulltext = JText::sprintf('PLG_CONTENT_SMARTSUB_ACCESS_DENIED_ACCOUNTS', $link);
				}
			}
			$row->text = $row->introtext . $row->fulltext;
		}
		
		return true;
	}
	
	/*
	 * Add "lock" icon to readmore button for articles that will require subscription
	 * upgrade for full text access
	 * 
	 * *** Easy way. It is possible to alter the whole readmore button link (with some
	 * help of javascript), so that it leads the user to registration / profile directly,
	 * but it will hit performance (need to calculate recommended accounts for multiple
	 * articles), so at the moment this feature is reserved...
	 */
	public function onContentBeforeDisplay($context, &$row, &$params, $page = 0)
	{
		$app = JFactory::getApplication();
		$canProceed = $app->isSite() && $context != 'com_content.article' && $this->componentInstalled;
		
		// if there is no fulltext, there is nothing to do
		if (!$canProceed || empty($row->fulltext))
		{
			return;
		}
		
		// no action for users outside Smartsub scope
		if(!$this->_isSmartsubScope())
		{
			return;
		}
		
		// Will subscription upgrade link be displayed in full article view?
		$user = JFactory::getUser();
		$authorizedGroups = JAccess::getAuthorisedViewLevels($user->get('id', 0));
		$attribs = json_decode($row->attribs);
		if(empty($attribs->fulltext_access) || in_array($attribs->fulltext_access, $authorizedGroups))
		{
			// article-global fulltext access or current user already has required view permissions
			return;
		}
		
		// This plugin is called after all "onContentPrepare" stuff, so we can rely on the row's
		// alternative_readmore property
		if(!empty($row->alternative_readmore))
		{
			// we have custom readmore, use it
			$row->alternative_readmore = '<span><i class="icon-lock"></i></span>' . $row->alternative_readmore;
		}
		else
		{
			// no custom readome, but we set it explicitly so that our lock icon will be displayed anyway
			// use default read more text
			$row->alternative_readmore = '<span><i class="icon-lock"></i></span>' . JText::_('COM_CONTENT_READ_MORE');
		}
		return;
	}
	
	private function _getRecommendedAccounts($target_level, $user_levels)
	{
		$db = JFactory::getDbo();
		
		$db->setQuery('SELECT rules FROM #__viewlevels WHERE id = ' . (int)$target_level);
		$rules = $db->loadResult();
		$groups = !empty($rules) ? json_decode($rules) : null;
		
		if(empty($groups))
		{
			// panic - we have a removed access level or an access level not linked to any group!
			return array();
		}
		
		$query = $db->getQuery(true);
		$query->select('a.id');
		$query->from('#__ss_accounts AS a');
		
		$query->where('a.group_id IN (SELECT c.id FROM #__usergroups AS b LEFT JOIN #__usergroups AS c ON c.lft BETWEEN b.lft AND b.rgt WHERE b.id IN(' . implode(',', $groups) . '))');
		// only show published accounts and also filter out those current user has no permissions to view
		$query->where('a.published = 1');
		$query->where('a.access IN(' . implode(',', $user_levels) . ')');
		
		$db->setQuery($query);
		try
		{
			$accounts = $db->loadColumn();
		}
		catch(RuntimeException $e)
		{
			JLog::add($e->getMessage(), JLog::WARNING, 'jerror');
			return array();
		}
		
		return $accounts;
	}
	
	private function _isSmartsubScope()
	{
		$user = JFactory::getUser();
		if(!$user->guest)
		{
			// we have to detect users created by admin with "Not a Smart Subscriptions Account" option
			$profile = JUserHelper::getProfile($user->get('id'));
			if(empty($profile->smartsub['account']))
			{
				return false;
			}
		}
		// guests are potential Smartsub users
		return true;
	}
}
