<?php
/**
 * @package plg_user_smartsub (Smart Subscriptions User Profile Plugin) for Joomla 3
 * @version 1.0
 * @author Alex Polonski
 * @copyright (C)2014 - Alex Polonski
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 **/
defined('JPATH_PLATFORM') or die;

require_once JPATH_ADMINISTRATOR . '/components/com_smartsub/helpers/smartsub.php';

class JFormFieldAccount extends JFormField
{
	protected $type = 'Account';
	protected $layout = 'default';

	protected function getInput()
	{
		$plugin = JPluginHelper::getPlugin('user', 'smartsub');
		$params = new JRegistry();
		$params->loadString($plugin->params);
		$this->layout = $params->get('accounts_layout', 'default');
		
		$app = JFactory::getApplication();
		
		$db = JFactory::getDbo();
		$query = $db->getQuery(true);
		$query->select('a.id, a.title, a.amount, a.currencyID, a.description, a.valid_period, a.valid_period_unit, a.is_default, a.published');
		$query->select('a.amount = 0 AS is_free');
		// select some more values as duplicates. In frontend they will be overwritten in getDiscounts() helper method,
		// but we need these values for backend - user manager profile form
		$query->select('a.amount as full_amount, 0 AS is_current, 0 AS discount, 0 AS discount_pct');
		$query->from('#__ss_accounts AS a');
		// skip accounts that do not refer to a valid user group
		$query->innerJoin('#__usergroups AS g ON g.id = a.group_id');
		if($app->isSite())
		{
			$query->where('a.published = 1');
			
			// for frontend we check access level of the user.
			$groups = JFactory::getUser()->getAuthorisedViewLevels();
			$query->where('a.access IN (' . implode(',', $groups) . ')');
			
			$has_user_data_in_session = 
				$app->getUserState('com_users.registration.data', null) ||
				$app->getUserState('com_users.edit.profile.data', null);
			$apply_recommended_accounts = $app->input->getInt('propose_acc', 0);
			
			if(!($apply_recommended_accounts) && !$has_user_data_in_session)
			{
				$app->setUserState('com_smartsub.recommended_accounts', null);
			}
			
			$recommended_accounts = $app->getUserState('com_smartsub.recommended_accounts', array());
			if(!empty($recommended_accounts))
			{
				JArrayHelper::toInteger($recommended_accounts);
				$query->where('a.id IN (' . implode(',', $recommended_accounts) . ')');
			}
		}
		else
		{
			// for admin use we allow unpublished accounts
			$query->where('a.published IN (0, 1)');
		}

		$query->order('a.ordering ASC');
		$db->setQuery($query);
		
		try 
		{
			$accounts = $db->loadObjectList();
		}
		catch(RuntimeException $e)
		{
			// log error here...
			return false;
		}
		
		if(empty($accounts))
		{
			return '<input type="text" readonly="readonly" class="readonly" value="' . JText::_('PLG_USER_SMARTSUB_NO_ACCOUNTS') . '"/>';
		}
		
		// get default account (if available)
		$default_account = '';
		foreach($accounts as $account)
		{
			if(!empty($account->is_default))
			{
				$default_account = $account->id;
				break;
			}
		}
		
		// get data from current user profile (if any)
		$user_id = $app->isSite() ? JFactory::getUser()->get('id', 0) : $this->form->getValue('id', 0);
		if($user_id)
		{
			$profile = JUserHelper::getProfile($user_id);
			$current_account_id = !empty($profile->smartsub['account']) ? $profile->smartsub['account'] : 0;
			$account_expires = !empty($profile->smartsub['account_expires']) ? $profile->smartsub['account_expires'] : false;
		}
		else
		{
			$current_account_id = 0;
			$account_expires = false;
		}
		
		// get all non-expired accounts with paid status for this user (if 'easy_account_switch' is ON
		// resulting array maximum size will be 1)
		$valid_accounts = SmartsubHelper::getCurrentAccounts($user_id);
		$valid_accounts_ids = array_keys($valid_accounts);
		
		$now_ts = JDate::getInstance()->getTimestamp();
		
		if($app->isSite())
		{
			// only apply discounts in frontend
			$accounts = SmartsubHelper::getDiscounts($accounts, $valid_accounts, $current_account_id, $now_ts);
		}
		
		$dispatcher = JDispatcher::getInstance();
		JPluginHelper::importPlugin('smartsub');
		
		$payment_links = $dispatcher->trigger('onGetPaymentLinkData');
		if(in_array(false, $payment_links, true))
		{
			// Plugin will have to raise its own error or throw an exception.
			return '<input type="text" readonly="readonly" class="readonly" value="' . JText::_('PLG_USER_SMARTSUB_NO_ACCOUNTS') . '"/>';
		}
		
		// add special submit input "free upgrade": used for payments with amount = 0 but for accounts that require smartsub
		// controller for correct logging (paid accounts that can be activated with 100% discount)
		$free_upgrade_link = '<input class="action-payment btn btn-success validate" type="submit" value="' . JText::_('PLG_USER_SMARTSUB_GRATIS') . '"/>';
		
		// check if current user account is about to expire and set "allow_renew" flag
		$allow_renew = false;
		if($account_expires)
		{
			// check expiration date
			$account_expires_ts = JDate::getInstance($account_expires)->getTimestamp();
			$expiration_data = SmartsubHelper::getExpirationDataById($profile->smartsub['account'], $now_ts);
		
			// if we are in expiration alert period, we pass a special flag to the client
			// script so that it will allow payment for all paid accounts, including the one
			// currently active
			if($account_expires_ts - $expiration_data['expiration_alert_time'] < $now_ts)
			{
				$allow_renew = true;
				$account_expires = JHtml::date($account_expires, JText::_('DATE_FORMAT_LC2'));
			}
		}
		
		// Start HTML output
		$html = array();
		
		// hidden value field - moved to "default" template, because "compact" template uses '<select />
		// element and doesn't require a hidden value field
		//$html[] = '<input type="hidden" name="' . $this->name .'" id="' . $this->id .'" value="' . $this->value .'" />';
		
		// add valid account ids to the form, so we will know when to display payment buttons
		$html[] = '<input type="hidden" id="pr-current-accounts" value="' . implode(',', $valid_accounts_ids) .'" />';
		
		// Inject current time stamp to the form, so that it can be used as checkout time for
		// discount calculation at the moment of order creation
		$html[] = '<input type="hidden" name="pr-checkout-ts" value="' . $now_ts .'" />';
		
		// get the rest of HTML (supports template overrides)
		$html[] = $this->getHTML($accounts, $allow_renew ? $account_expires : false, $app->isAdmin());
		
		// include client script
		$doc = JFactory::getDocument();
		$doc->addScript(JUri::root() . 'plugins/user/smartsub/smartsub.js');
		
		$input = $app->input;
		$view = $input->getCmd('view', 'registration');
		$options = array(
				'is_admin' => $app->isAdmin(),
				'view' => $view,
				'value' => $this->value,
				'value_field_id' => $this->id,
				'payment_links_data' => $payment_links,
				'free_upgrade_link' => $free_upgrade_link,
				'allow_renew_btn' => JText::_('PLG_USER_SMARTSUB_ALLOW_RENEW', true), // "account upgrade2 text
				'avoid_renew_btn' => JText::_('PLG_USER_SMARTSUB_AVOID_RENEW', true), // "no thanks" text
				'allow_renew' => $allow_renew,
				// used for show/hide account expiration control for admins in user edit form
				'expiration_override_field_id' => $this->form->getFormControl() . '_smartsub_account_expires',
				'single_account_description' => count($accounts) > COM_SMARTSUB_MAX_ACCOUNT_DESCRIPTIONS_VISIBLE,
				'default_account' => $default_account,
				'layout' => $this->layout,
				// default form action for profile: compatibility with 3-d party components (e.g. community builder)
				'def_profile_action' => JRoute::_('index.php?option=com_smartsub&task=free.profile', false)
		);
		$doc->addScriptDeclaration('
				jQuery(document).ready(function() {
					plg_smartsub.init(' . json_encode($options) . ');
				});
		');
		
		// include style
		$doc->addStyleSheet(JUri::root() . 'plugins/user/smartsub/smartsub.css');
		
		return implode("\n", $html);
	}
	
	protected function getLabel()
	{
		return parent::getLabel();
	}
	
	protected function getHTML($accounts, $account_expires_soon, $isAdmin)
	{
		$layout_path = JPluginHelper::getLayoutPath('user', 'smartsub', $this->layout);
		
		// for backend use we can still rely on template override, but if no override
		// is found, we must fallback to layout file from plugin itself.
		// *** getLayoutPath() will not work for plugins, as it will return a path
		// based on JPATH_BASE - we need JPATH_ROOT here
		if($isAdmin && !file_exists($layout_path))
		{
			$layout_path = JPATH_ROOT . '/plugins/user/smartsub/tmpl/' . $this->layout . '.php';
		}
		
		// Start capturing output into a buffer
		ob_start();
		
		// Include the requested template filename in the local scope
		// (this will execute the view logic).
		include $layout_path;
		
		// Done with the requested template; get the buffer and
		// clear it.
		$output = ob_get_contents();
		ob_end_clean();
		
		return $output;
	}
}
