/**
 * @package plg_user_smartsub (Smart Subscriptions User Profile Plugin) for Joomla 3
 * @version 1.2.3
 * @author Alex Polonski
 * @copyright (C)2014 - Alex Polonski
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 **/
(function($) {
	plg_smartsub = {
		options : {
			is_admin : 0,
			view : 'registration',
			value : '',
			value_field_id : '',
			payment_links_data : [],
			allow_renew : 0, // will be set to 1 if current account is going to expire soon
			expiration_override_field_id : '',
			selected_class : 'btn-success',
			allow_renew_btn : 'Update subscription',
			avoid_renew_btn : 'No, thanks',
			single_account_description : 0,
			default_account : '',
			layout : 'default',
			def_profile_action : ''
		},
		default_elements : {
			form_action : '',
			task : 'registration.register',
			option : 'com_users'
		},
		form : null,
		init : function(options) {
			this.options = $.extend(this.options, options);
			
			if(this.options.is_admin == 0) {
				// we need this stuff for frontend only
				
				// look for K2 user registration / profile forms
				if($('form[name="josForm"]').length) {
					this.form = $('form[name="josForm"]');
				} else if($('form[name="userform"]').length) {
					this.form = $('form[name="userform"]');
				} else {
					// core Joomla form
					this.form = $("#member-registration").length ? $("#member-registration") : $("#member-profile");
				}
				
				if(this.options.view == 'registration') {
					// use Joomla com_users core values for registration
					this.default_elements.form_action = this.form.attr("action");
					this.default_elements.task = this.form.find("input[name='task']").val();
					this.default_elements.option = this.form.find("input[name='option']").val();
				} else {
					// use Smartsub defaults for profile
					this.default_elements.form_action = this.options.def_profile_action;
					this.default_elements.task = "free.profile";
					this.default_elements.option = "com_smartsub";
				}
				
				/* Old version - always use Joomla com_users core values
				this.default_elements.form_action = this.form.attr("action");
				this.default_elements.task = this.form.find("input[name='task']").val();
				this.default_elements.option = this.form.find("input[name='option']").val();
				*/
				
				// in frontend we preselect default account if no account is assigned to the user
				// (typical for new registrations)
				if(this.options.value == '') {
					this.options.value = this.options.default_account;
				}
			}
			
			var self = this;

			if(this.options.layout == "compact") { // accounts as a dropdown
				this.setSelected(this.options.value);
				
				$("select.action-account").on("change", function() {
					self.setSelected($(this).val());
					if(self.options.is_admin == 0) {
						self.setRegisterButtons();
					}
				});
			} else { // accounts as <a /> elements
				this.setSelected(this.options.value);
				
				$("a.action-account").on("click", function(ev) {
					var value = $(ev.target).closest("a.action-account").attr('data-account-id');
					self.setSelected(value);
					if(self.options.is_admin == 0) {
						self.setRegisterButtons();
					}
				});
			}
			if(this.options.is_admin == 0) {
				this.setRegisterButtons();
			}
			
			// this will have effect only if .action-allow-renew button is
			// rendered on the page
			$("button.action-allow-renew").on("click", function(ev) {
				var button = $(ev.target);
				// toggle allow_renew option
				self.options.allow_renew = !self.options.allow_renew;
				button.toggleClass("btn-primary");
				// update button text
				if(self.options.allow_renew == 0) {
					button.text(self.options.allow_renew_btn);
					// hide expiration notice
					/* $$$ consider refactoring DOM - we may use a class "action-allow-renew-wrapper"... 
					 * and also give p element a class, so that we do not depend on layout */
					$("#allow-renew-wrapper p").hide();
				} else {
					button.text(self.options.avoid_renew_btn);
					// show expiration notice
					/* $$$ consider refactoring DOM - we may use a class "action-allow-renew-wrapper"... 
					 * and also give p element a class, so that we do not depend on layout */
					$("#allow-renew-wrapper p").show();
				}
				// redraw buttons
				self.setRegisterButtons();
			});
		},
		getSelected : function() {
			if(this.options.layout == "compact") {  // accounts as a dropdown
				return $("select.action-account").children("option:selected");
			} else {  // accounts as <a /> elements
				return $("a.action-account[data-account-id='" + $("#" + this.options.value_field_id).val() + "']");
			}
		},
		setSelected : function(account_id) {
			var active_account;
			if(this.options.layout == "compact") {  // accounts as a dropdown
				active_account = $("select.action-account").find("option[value='" + account_id + "']");
				if(active_account.length == 0) {
					active_account = $("select.action-account option").first();
					account_id = active_account.attr("data-account-id");
				}
				$("select.action-account").val(account_id);
				$("select.action-account").trigger("liszt:updated");
				
				if(this.options.is_admin == 0 && $("select.action-account option").length == 1) {
					$("select.action-account option").closest("div").hide();
				}
			} else {
				$("a.action-account").removeClass(this.options.selected_class);
				
				// shortcut to selected account
				active_account = $("a.action-account[data-account-id='" + account_id + "']");
				if(active_account.length == 0) {
					// current account is not available, select the first available account
					// as preselected
					active_account = $("a.action-account").first();
					account_id = active_account.attr("data-account-id");
				}
				active_account.addClass(this.options.selected_class);
				
				// set form field account_id value
				$("#" + this.options.value_field_id).val(account_id);
				
				// if only 1 account is available for selection, no need to show account selection
				// button. The rest of the logic will not change (for frontend)
				if(this.options.is_admin == 0 && $("a.action-account").length == 1) {
					$("a.action-account").hide();
				}
			}
			if(this.options.is_admin == 1) {
				// for admin we hide account expiration input for free and custom accounts
				// Also disable the field, so its data is not sent in form
				if(active_account.attr("data-account-paid") == 0) {
					$("#" + this.options.expiration_override_field_id).closest("div.control-group").hide();
				} else {
					$("#" + this.options.expiration_override_field_id).closest("div.control-group").show();
				}
			}
			
			if(this.options.single_account_description == 1) {
				$(".account-info").hide();
				$(".account-info[id='account-info-" + account_id + "']").show();
			}
		},
		setRegisterButtons : function() {
			var core_submit_btn = this.form.find("button[type='submit']").not(".action-payment");
			var selected = this.getSelected();
			
			// we allow free selection between currently active accounts
			var current_accounts = $("#pr-current-accounts").val();
			current_accounts = (current_accounts != "" ? current_accounts.split(',') : []);
			// Only allow renew if set in global options and if we are dealing the the same account
			var allow_renew = this.options.allow_renew == 1 && selected.attr("data-account-id") == this.options.value;
			if(selected.attr("data-default-apply") != 1 &&
					(current_accounts.indexOf(selected.attr("data-account-id")) == -1 || allow_renew)) {
				
				// an account that requires payment was selected: we have to replace
				// standard form action and request control field values with custom
				// ones
				core_submit_btn.hide();
				
				// clear existing payment submit inputs
				$(".action-payment").remove();
				
				if(selected.attr("data-account-paid") == 0) {
					// free upgrade submit - only free upgrade button display
					core_submit_btn.after(this.options.free_upgrade_link);
				} else {
					// display all available payment submit inputs
					var i;
					for(i = 0; i < this.options.payment_links_data.length; i++) {
						core_submit_btn.after(this.options.payment_links_data[i].html);
					}
				}
				
				// some payment buttons can define "data-currency-id" attribute, meaning
				// this a payment method is valid only for a limited set of currencies.
				// We have to hide a payment button if account selected uses incompatible
				// currency
				$(".action-payment").each(function() {
					$this = $(this);
					var allowed_currency = $this.attr("data-currency-id");
					if(allowed_currency && $.inArray(selected.attr("data-currency-id"), allowed_currency.split(',')) < 0) {
						$this.hide();
					} else {
						$this.show();
					}
				});
				
				// set custom task and option (at the moment there is no need to set
				// these values depending on the payment method selected, but we can
				// do it if nescessary - just move the lines below inside input onclick
				// event handler)
				
				var task = this.options.view == 'registration' ? 
						'payment.register' : 'payment.profile';
				this.form.find("input[name='task']").val(task);
				this.form.find("input[name='option']").val("com_smartsub");
				
				var self = this;
				$(".action-payment").on("click", function(ev) {
					// set form action depending on the payment button clicked
					// *** another solution: inject "payment-method" hidden field
					// into the form and set it here. Form action would become
					// common for all payments methods and could be set along with
					// "task" and "option"
					var payment_method = $(ev.target).attr("data-payment-method");
					var clicked = $.grep(self.options.payment_links_data, function(v) {
						return v.payment_method == payment_method;
					});
					
					if(clicked.length) {
						clicked = clicked.pop();
						self.form.attr("action", clicked.action);
					}
				});
			} else {
				// an account that doesn't require payment was selected: standard
				// form action and request control field values will work OK.
				// Restore initial form configuration
				core_submit_btn.show();
				$(".action-payment").remove();
				this.form.find("input[name='task']").val(this.default_elements.task);
				this.form.find("input[name='option']").val(this.default_elements.option);
				this.form.attr("action", this.default_elements.form_action);
			}
			
			// If currently selected account is not the one from user profile
			// no need to show "allow-renew" notice and button
			// (if this block is not rendered on the page, the code below
			// will have no effect)
			
			if($("#allow-renew-wrapper").length > 0) {
				if(selected.attr("data-account-id") != this.options.value) {
					$("#allow-renew-wrapper").hide();
				} else {
					$("#allow-renew-wrapper").show();
				}
				// for now the elements visibility inside allow-renew div is
				// controlled in the button's click handler...
				/* $$$ consider refactoring DOM - we may use a class "action-allow-renew-wrapper"... 
				 * and also give p element a class, so that we do not depend on layout
				if(this.options.allow_renew == 0) {
					$("#allow-renew-wrapper p").hide();
					$(".action-allow-renew").text(this.options.allow_renew_btn);
				} else {
					$("#allow-renew-wrapper p").show();
					$(".action-allow-renew").text(this.options.avoid_renew_btn);
				}
				*/
			}
			/*
			if($(".action-allow-renew").length > 0) {
				if(selected.attr("data-account-id") != this.options.value) {
					$(".action-allow-renew").closest("div").hide();
				} else {
					$(".action-allow-renew").closest("div").show();
				}
			}
			*/
		}
	};
})(jQuery.noConflict());