<?php
/**
 * @package plg_user_smartsub (Smart Subscriptions User Profile Plugin) for Joomla 3
 * @version 1.1.8
 * @author Alex Polonski
 * @copyright (C)2014 - Alex Polonski
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 **/
defined('JPATH_BASE') or die;

if(file_exists(JPATH_ADMINISTRATOR . '/components/com_smartsub/helpers/smartsub.php'))
{
	require_once(JPATH_ADMINISTRATOR . '/components/com_smartsub/helpers/payment.php');
	require_once(JPATH_ADMINISTRATOR . '/components/com_smartsub/helpers/smartsub.php');
}

class PlgUserSmartsub extends JPlugin
{
	/**
	 * Load the language file on instantiation.
	 *
	 * @var    boolean
	 * @since  3.1
	 */
	
	protected $autoloadLanguage = true;
	
	// Smartsub component presense flag
	protected $componentInstalled = false;

	/**
	 * Constructor
	 *
	 * @param   object  $subject  The object to observe
	 * @param   array   $config   An array that holds the plugin configuration
	 *
	 * @since   1.5
	 */
	public function __construct(& $subject, $config)
	{
		parent::__construct($subject, $config);
		JFormHelper::addFieldPath(__DIR__ . '/fields');
		
		$this->componentInstalled = JComponentHelper::isEnabled('com_smartsub');
	}

	/**
	 * @param   string     $context  The context for the data
	 * @param   integer    $data     The user id
	 *
	 * @return  boolean
	 *
	 * @since   1.6
	 */
	public function onContentPrepareData($context, $data)
	{
		if(!$this->componentInstalled)
		{
			return true;
		}
		
		// Check we are manipulating a valid form.
		
		if (!in_array($context, array('com_users.user', 'com_users.profile')))
		{
			return true;
		}
		
		if (is_object($data))
		{
			$userId = isset($data->id) ? $data->id : 0;
			
			$db = JFactory::getDbo();
			if (!isset($data->smartsub) and $userId > 0)
			{
				// Load the profile data from the database.
				$db = JFactory::getDbo();
				$db->setQuery(
					'SELECT profile_key, profile_value FROM #__user_profiles' .
						' WHERE user_id = ' . (int) $userId . " AND profile_key LIKE 'smartsub.%'" .
						' ORDER BY ordering'
				);

				try
				{
					$results = $db->loadObjectList('profile_key');
				}
				catch (RuntimeException $e)
				{
					$this->_subject->setError($e->getMessage());
					return false;
				}

				$data->smartsub = array();

				foreach ($results as $k => $v)
				{
					$k = str_replace('smartsub.', '', $k);
					$data->smartsub[$k] = json_decode($v->profile_value, true);
					
					if ($data->smartsub[$k] === null)
					{
						$data->smartsub[$k] = $v->profile_value;
					}
				}
			}
			
			// get account expiration from pr_payments table
			if(!empty($data->smartsub['account']))
			{
				// at this point $data->smartsub['account'] may hold an ID or
				// account description (already prepared for profile info view)
				// Just look in $data->smartsub['account_id'] property and use it if set
				$account_id = !empty($data->smartsub['account_id']) ? (int)$data->smartsub['account_id'] : (int)$data->smartsub['account'];
				
				$db->setQuery('SELECT expires FROM #__ss_payments WHERE user_id = '
						. (int)$userId . ' AND account_id = '
						. $account_id . ' AND status = '
						. $db->Quote('paid') . ' ORDER BY date DESC LIMIT 1');
				try
				{
					$account_expires = $db->loadResult();
				}
				catch(RuntimeException $e)
				{
					$account_expires = false;
				}
				
				if(isset($account_expires))
				{
					$data->smartsub['account_expires'] = intval($account_expires) ? $account_expires : false;
				}
			}
		}
		
		// For frontend profile view (non-edit) we have to display
		// smartsub account title, not account_id
		$input = JFactory::getApplication()->input;
		$option = $input->getCmd('option');
		$view = $input->getCmd('view');
		$layout = $input->getCmd('layout');
		
		// *** Take care: if layout is undefined the code block below can be executed...
		if($option == 'com_users' && (($view == 'profile' && $layout != 'edit') || $view == 'user'))
		{
			if(!empty($data->smartsub['account']))
			{
				$account_id = (int)$data->smartsub['account'];
				// if $data->smartsub['account'] is not numeric, look in temporary 'account_id' property
				if(empty($account_id))
				{
					$account_id = (int)$data->smartsub['account_id'];
				}
			
				JTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_smartsub/tables');
				$account_table = JTable::getInstance('Account', 'SmartsubTable');
				if($account_table->load($account_id))
				{
					$title = $account_table->title;
					if(!empty($data->smartsub['account_expires']) && $data->smartsub['account_expires'] != $db->getNullDate())
					{
						// show expiration date (if any)
						$title .= JText::sprintf('PLG_USER_SMARTSUB_ACCOUNT_EXPIRES', JHtml::date($data->smartsub['account_expires'],  JText::_('DATE_FORMAT_LC2'), true));
					}
					else 
					{
						$title .= ' (' . JText::_('PLG_USER_SMARTSUB_ACCOUNT_NEVER_EXPIRES') . ')';
					}
				}
			}
			if(!empty($title))
			{
				if($view == 'user')
				{
					// for backend user manager - add account description field
					$data->smartsub['account_desc'] = $title;
				}
				else 
				{
					// add temp replacement for account_id property: *** we cannot
					// use custon field name for account description as it is intended
					// for Joomla core profile view
					$data->smartsub['account_id'] = $account_table->id;
					$data->smartsub['account'] = $title;
				}
			}
			else
			{
				unset($data->smartsub['account']);
				unset($data->smartsub['account_id']);
			}
		}

		return true;
	}

	/**
	 * @param   JForm    $form    The form to be altered.
	 * @param   array    $data    The associated data for the form.
	 *
	 * @return  boolean
	 * @since   1.6
	 */
	public function onContentPrepareForm($form, $data)
	{
		if(!$this->componentInstalled)
		{
			return true;
		}
		
		if (!($form instanceof JForm))
		{
			$this->_subject->setError('JERROR_NOT_A_FORM');
			return false;
		}

		// Check we are manipulating a valid form.
		
		$name = $form->getName();
		if (!in_array($name, array('com_users.user', 'com_users.registration', 'com_users.profile')))
		{
			return true;
		}
		
		// for frontend profile edit view we have to check for users without account setting
		// in their profiles (users added manually by administrator)
		if($name == 'com_users.profile')
		{
			$user = JFactory::getUser();
			$profile = JUserHelper::getProfile($user->get('id'));
			if(empty($profile->smartsub['account']))
			{
				return true;
			}
		}
		
		// if there are no published accounts, nothing to do
		$db = JFactory::getDbo();
		$db->setQuery('SELECT COUNT(id) FROM #__ss_accounts WHERE published IN(0,1)');
		$count = $db->loadResult();
		if(empty($count))
		{
			return true;
		}
		
		$is_frontend = ($name == 'com_users.registration' || $name == 'com_users.profile');
		
		JHtml::_('jquery.framework');
		JForm::addFormPath(__DIR__ . '/profiles');
		
		if($is_frontend)
		{
			// Add the registration fields and supporting script to
			// frontend registration form
			$form->loadFile('frontprofile', false);
			
			// if 'hide_profile_fields' option is active, add JS
			if($this->params->get('hide_profile_fields', 0))
			{
				// Only 'account' selection control will be displayed
				JFactory::getDocument()->addScriptDeclaration('
						jQuery(document).ready(function($) {
							var fieldset = $("#pr-current-accounts").closest("fieldset");
							// Hiding fieldsets seems to do the trick: invisible controls data is still
							// present on the form, so there are no "missing fields" errors, but are not
							// accessible via keyboard or tabstop.
							$("#member-profile").find("fieldset").not(fieldset).hide();
						});
				');
			}
			if(COM_SMARTSUB_DEMO_MODE == 1)
			{
				$form->setFieldAttribute('account', 'description', JText::_('PLG_USER_SMARTSUB_ACCOUNT_DEMO_DESC'), 'smartsub');
			}
		}
		else 
		{
			// in backend we have to display accounts selection only for new users
			// or users with no smartsub profile
			$tmp = (object)$data;
			$userId = empty($tmp->id) ? false : $tmp->id;
			$account = false;
			if($userId)
			{
				$profile = JUserHelper::getProfile($userId);
				$account = empty($profile->smartsub['account']) ? false : $profile->smartsub['account'];
			}
			if($account)
			{
				// Add special form with account hidden and a prompt for
				// paid registration payments view for account management
				$form->loadFile('adminprofile', false);
			}
			else
			{
				// Add the smartsub account fields to the user backend form.
				$form->loadFile('profile', false);
			}
		}
		
		return true;
	}

	/**
	 * Method is called before user data is stored in the database
	 *
	 * @param   array    $user   Holds the old user data.
	 * @param   boolean  $isnew  True if a new user is stored.
	 * @param   array    $data   Holds the new user data.
	 *
	 * @return    boolean
	 *
	 * @since   3.1
	 * @throws    InvalidArgumentException on invalid date.
	 */
	public function onUserBeforeSave($user, $isNew, $data)
	{	
		return true;
	}

	public function onUserAfterSave($data, $isNew, $result, $error)
	{
		if(!$this->componentInstalled)
		{
			return true;
		}
		
		$userId = JArrayHelper::getValue($data, 'id', 0, 'int');
		
		/* 
		 * *** We are working now in custom namespace in profile ("smartsub.%"), so there is
		 * no real need to check if isset($data['smartsub']), so that if smartsub profile data
		 * is totally missing we still remove all related profile keys.
		 */
		if ($userId && $result && isset($data['smartsub']) && (count($data['smartsub'])))
		{
			// Protect a super user from assigning Smartsub profile
			$isUserSuperAdmin = JFactory::getUser($userId)->authorise('core.admin');
			if($isUserSuperAdmin && !empty($data['smartsub']['account']))
			{
				JError::raiseNotice('403', JText::_('PLG_USER_SMARTSUB_WARNING_SUPERADMIN'));
				// unset smartsub-related profile keys, so that the rest of profile data
				// can be saved successfully
				unset($data['smartsub']['account']);
			}
			try
			{
				$app = JFactory::getApplication();
				
				$db = JFactory::getDbo();
				$query = $db->getQuery(true)
					->delete($db->quoteName('#__user_profiles'))
					->where($db->quoteName('user_id') . ' = ' . (int) $userId)
					->where($db->quoteName('profile_key') . ' LIKE ' . $db->quote('smartsub.%'));
				$db->setQuery($query);
				$db->execute();

				$tuples = array();
				$order = 1;

				foreach ($data['smartsub'] as $k => $v)
				{
					if($k == 'account' && empty($data['smartsub']['account']))
					{
						// do not add empty value to profiles table
						unset($data['smartsub']['account']);
					}
				}
				
				if(!empty($data['smartsub']['account']))
				{
					// load account data
					JTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_smartsub/tables');
					$account_table = JTable::getInstance('Account', 'SmartsubTable');
					$account_id = $data['smartsub']['account'];
					if(!$account_table->load($account_id))
					{
						// may be an obsolete account_id. Remove it from profile
						unset($data['smartsub']['account']);
					}
					
					$component = $app->input->getCmd('option', '');
					
					if(JFactory::getApplication()->isAdmin() && $component == 'com_users')
					{
						// for administration user management insert row in pr_payments table for paid accounts
						// only if it comes from a form with "is_new_payment" hidden field non-empty.
						if(!empty($account_table->id) && $account_table->amount > 0 && !empty($data['smartsub']['is_new_payment']))
						{
							// prepare new row for insert
							$row = array(
									'user_id' => $userId,
									'amount' => $account_table->amount,
									'status' => 'paid',
									'currencyID' => $account_table->currencyID,
									'transactionID' => SmartsubHelper::getInternalTransactionID($data['username'], $account_id, 'ADMIN'),
									'order_time' => JDate::getInstance()->toSql(), // $$$ not implemented in DB!
									'date' => JDate::getInstance()->toSql(),
									'system' => 'administrative',
									// set task based on $isNew
									'task' => $isNew ? 'register' : 'profile',
									'account_id' => $account_id,
									// for admins in user manager we accept expired date from user edit form
									'expires' => empty($data['smartsub']['account_expires']) ? $db->getNullDate() : $data['smartsub']['account_expires']
							);

							$payment = JTable::getInstance('Payment', 'SmartsubTable');
							
							if(!$payment->save($row))
							{
								// contact administrator!
								$this->_subject->setError(JText::_('PLG_USER_SMARTSUB_PAYMENT_LOGGING_FAILED'), 'warning');
								return false;
							}
						}
					}
					// check if we are saving the user with a free account in profile
					if(!empty($account_table) && $account_table->amount == 0)
					{
						$payment = JTable::getInstance('Payment', 'SmartsubTable');
						$row = array(
								'user_id' => $userId,
								'amount' => $account_table->amount,
								'status' => 'paid',
								'currencyID' => $account_table->currencyID,
								'transactionID' => SmartsubHelper::getInternalTransactionID($data['username'], $account_id, 'FREE'),
								'order_time' => JDate::getInstance()->toSql(), // $$$ not implemented in DB!
								'date' => JDate::getInstance()->toSql(),
								'system' => 'none',
								// set task based on account_reset flag and $isNew
								'task' => empty($data['smartsub']['account_reset']) ? ($isNew ? 'register' : 'profile') : $data['smartsub']['account_reset'],
								'offline' => 0, // we mark free account records as NOT offline, so that status cannot be edited in payments list view
								'account_id' => $account_id,
								// if temp old_account is set in profile, use it as upgrade_from
								'upgrade_from' => !empty($data['smartsub']['old_account']) ? $data['smartsub']['old_account'] : false,
								'expires' => $db->getNullDate()
						);
						
						// when saving a free account always add/update row in payments table, so that 'easy_accounts_switch'
						// setting is respected
						if($payment->load(array('user_id' => $row['user_id'], 'account_id' => $account_id, 'status' => 'paid')))
						{
							// if we find existing paid row for this user and account, do not alter row data. Saving the row
							// serves as a trigger to 'easy_accounts_switch' control. This is not the best solution, 
							// check if we can move 'easy_accounts_switch' stuff to a dedicated method...
							$row['id'] = $payment->id;
							unset($row['task']);
							unset($row['upgrade_from']);
							unset($row['date']);
						}
						if(!$payment->save($row))
						{
							// contact administrator!
							$this->_subject->setError(JText::_('PLG_USER_SMARTSUB_PAYMENT_LOGGING_FAILED'), 'warning');
							return false;
						}
					}
				}
				else 
				{
					// a user without Smartsub profile is saved. If he previousely had any 'paid'
					// subscriptions we have to clean up payments table - set 'archived' status
					// for all 'paid' transactions for this user
					try
					{
						$db->setQuery('UPDATE #__ss_payments SET status = ' . $db->Quote('archived') . ' WHERE user_id = ' . $userId . ' AND status = ' . $db->Quote('paid'));
						$db->execute();
					}
					catch(RuntimeException $e)
					{
						// this is not an error. Component may not be installed at all (which is not normal,
						// but still possible)
					}
				}
				
				// do not save account_expires in profile, it will be read from pr_payments table
				unset($data['smartsub']['account_expires']);
				// clean up the new_payment flag after use - it is not intended to be saved in profile
				unset($data['smartsub']['is_new_payment']);
				// clean up account reset task and old account
				unset($data['smartsub']['account_reset']);
				unset($data['smartsub']['old_account']);
				
				foreach ($data['smartsub'] as $k => $v)
				{
					$tuples[] = '(' . $userId . ', ' . $db->quote('smartsub.' . $k) . ', ' . $db->quote(json_encode($v)) . ', ' . $order++ . ')';
				}
				
				if(!empty($tuples))
				{
					$db->setQuery('INSERT INTO #__user_profiles VALUES ' . implode(', ', $tuples));
					$db->execute();
				}
				
				// clean up recommended accounts from session after user is successfully saved
				// *** support for smartsub content plugin ("readmore access")
				$app->setUserState('com_smartsub.recommended_accounts', null);
			}
			catch (RuntimeException $e)
			{
				$this->_subject->setError($e->getMessage());
				return false;
			}
			
			// force group assignment if paid registration account is set
			$group = !empty($account_table) ? $account_table->group_id : false;
			
			if(!empty($group))
			{
				if(!self::_setUserGroup($userId, $group))
				{
					// log error
					return false;
				}
			}
		}

		return true;
	}

	public function onUserBeforeDelete($user)
	{
		return true;
	}
	
	/**
	 * Remove all user profile information for the given user ID
	 *
	 * Method is called after user data is deleted from the database
	 *
	 * @param   array    $user     Holds the user data
	 * @param   boolean  $success  True if user was succesfully stored in the database
	 * @param   string   $msg      Message
	 *
	 * @return  boolean
	 */
	public function onUserAfterDelete($user, $success, $msg)
	{
		if(!$this->componentInstalled)
		{
			return true;
		}
		
		if (!$success)
		{
			return false;
		}

		$userId = JArrayHelper::getValue($user, 'id', 0, 'int');

		if ($userId)
		{
			try
			{
				$db = JFactory::getDbo();
				$db->setQuery(
					'DELETE FROM #__user_profiles WHERE user_id = ' . $userId .
						" AND profile_key LIKE 'smartsub.%'"
				);
				$db->execute();
				
				$db->setQuery(
						'DELETE FROM #__ss_payments WHERE user_id = ' . $userId
				);
				$db->execute();
			}
			catch (Exception $e)
			{
				$this->_subject->setError($e->getMessage());
				return false;
			}
		}

		return true;
	}
	
	public function onUserLogin($user, $options = array())
	{
		if(!$this->componentInstalled)
		{
			return true;
		}
		
		$app = JFactory::getApplication();
		if($app->isAdmin())
		{
			// no action in backend
			return true;
		}
		
		// clean up recommended accounts from session after user is successfully saved
		// *** support for smartsub content plugin ("readmore access")
		$app->setUserState('com_smartsub.recommended_accounts', null);
	
		$instance = JUser::getInstance();
		$id = (int) JUserHelper::getUserId($user['username']);

		if($id)
		{
			$instance->load($id);
		}
		else
		{
			return true;
		}
		
		if($instance->get('block', 0) == 1)
		{
			// no action if account is blocked
			return true;
		}
		
		$profile = JUserHelper::getProfile($instance->get('id'));
		
		// For profile info view 'account' property in user profile will be replaced
		// by profile title and expiration date (for presentation purposes)
		// If this is the case, a temporary 'account_id' property is added, so we
		// check it here if 'account' is not an integer
		
		if(!empty($profile->smartsub['account']))
		{
			$account_id = (int)$profile->smartsub['account'];
			if(empty($account_id))
			{
				// account is a string, try account_id
				$account_id = !empty($profile->smartsub['account_id']) ? (int)$profile->smartsub['account_id'] : false;
			}
		}
		else 
		{
			$account_id = false;
		}
		unset($profile->smartsub['account_id']);
		
		if(empty($account_id))
		{
			// no action for users without account in their profiles unless 'auto_free_account'
			// option is set and a free account is not found
			if(($free_account = SmartsubHelper::findFreeAccount()) && $this->params->get('auto_free_account', 0))
			{
				$groups = $instance->get('groups');
				// only act if current user is a member of only one group and it is the
				// same group as the one linked to the free account. *** this is required to
				// guarantee that other users (e.g. authors and editors) will not be modified
				// on frontend login.
				if(count($groups) == 1 && $groups[0] == static::_getAccountGroup($free_account))
				{
					$profile = JUserHelper::getProfile($id);
					$profile->smartsub['account'] = $free_account;
				
					static::_resetToFreeAccount($instance, $profile, null, 'register');
				}
			}
			return true;
		}
		
		// get most recent row from pr_payments table with 'paid' status for this user and current account_id.
		// Ignoring 'pending' and 'error' states allows a user to pass account check on login if he already has
		// a valid paid account (with payment in 'paid' state), but there is a more recent record waiting for
		// admin's confirmation in paymnents table - it is important for offline payment methods!
		$db = JFactory::getDbo();
		$query = $db->getQuery(true);
		$query->select('a.id, a.amount, a.valid_period, a.valid_period_unit')
			->from('#__ss_accounts AS a');
		$query->select('p.expires, p.status')
			->join('LEFT OUTER', '#__ss_payments AS p ON p.account_id = a.id AND p.status = ' . $db->Quote('paid') . ' AND p.user_id = ' . (int)$id);
		$query->where('a.id = ' . (int)$account_id);
		$query->order('p.date DESC');
		$query->limit(1);
		$db->setQuery($query);
		try
		{
			$result = $db->loadObject();
			if(empty($result))
			{
				throw new Exception('Invalid account ID');
			}
		}
		catch(Exception $e)
		{
			$app->enqueueMessage($e->getMessage(), 'warning');
			$app->redirect($options['return']);
			return true;
		}
		
		$profile_link = JRoute::_('index.php?option=com_users&view=profile&layout=edit&id=' . $instance->get('id'), false);
		
		// check if payment was valid
		if(empty($result->status) || $result->status != 'paid')
		{
			// reset to free account
			if(self::_resetToFreeAccount($instance, $profile, $account_id, 'block'))
			{
				$app->enqueueMessage(JText::sprintf('PLG_USER_SMARTSUB_INVALID_ACCOUNT_RESET', $profile_link), 'notice');
				// no need to abort login process because user is updated in session
				// automatically on save, so valid view levels will be applied
			}
			else
			{
				// no free account, add message
				$app->enqueueMessage(JText::_('PLG_USER_SMARTSUB_INVALID_ACCOUNT_BLOCK'), 'warning');
				// display message and redirect to home, that will guarantee
				// that login process is aborted
				$app->redirect($options['return']);
			}
			return true;
		}
		
		// check if account is not expired
		$expires_ts = intval($result->expires) > 0 ? JDate::getInstance($result->expires)->getTimestamp() : false;
		if(!$expires_ts)
		{
			// no more checks if account never expires
			return true;
		}
		
		$expiration_data = SmartsubHelper::getExpirationData($result->valid_period, $result->valid_period_unit);
		$now_ts = $expiration_data['now_ts'];
			
		if($expires_ts - $expiration_data['expiration_alert_time'] > $now_ts)
		{
			// not expired and not expiring soon
			return true;
		}
		elseif($expires_ts < $now_ts)
		{
			// reset to free account
			if(self::_resetToFreeAccount($instance, $profile, $account_id, 'expired'))
			{
				$app->enqueueMessage(JText::sprintf('PLG_USER_SMARTSUB_EXPIRED_ACCOUNT_RESET', $profile_link), 'notice');
				// no need to abort login process because user is updated in session
				// automatically on save, so valid view levels will be applied
			}
			else
			{
				// no free account found, just display the warning
				$app->enqueueMessage(JText::_('PLG_USER_SMARTSUB_EXPIRED_ACCOUT_DENIED'), 'warning');
				// display message and redirect to home, that will guarantee
				// that login process is aborted
				$app->redirect($options['return']);
			}
			return true;
		}
		else
		{
			// alert user
			$expired_date = JHtml::date($expires_ts, JText::_('DATE_FORMAT_LC2'));
			$app->enqueueMessage(JText::sprintf('PLG_USER_SMARTSUB_ACCOUNT_EXPIRATION_ALERT', $expired_date, $profile_link), 'message');
			return true;
		}
	}
	
	/*
	 * Protect user group(s) from deletion if it is linked with at least one Smart Subscriptions account
	 */
	public function onUserBeforeDeleteGroup($data)
	{
		$app = JFactory::getApplication();
		$canProceed = $app->isAdmin() && $this->componentInstalled;
		
		// if there is no fulltext, there is nothing to do
		if (!$canProceed)
		{
			return;
		}
		
		// Get number of accounts linked to the group being deleted or to one of its children
		// (child groups are deleted automatically with the parent group so we have to check them also) 
		$group_id = $data['id'];
		$db = JFactory::getDbo();
		$db->setQuery('SELECT COUNT(a.id) FROM #__ss_accounts AS a INNER JOIN #__usergroups AS b ON b.id = a.group_id AND b.lft BETWEEN ' . $data['lft'] . ' AND ' . $data['rgt']);
		try 
		{
			$count = $db->loadResult();
		}
		catch(RuntimeException $e)
		{
			JLog::add($e->getMessage(), JLog::WARNING, 'jerror');
			$count = 0;
		}
		
		if($count)
		{
			// return value is ignored when this plugin is called, so we have to force a redirect with message
			$app->enqueueMessage(JText::sprintf('PLG_USER_SMARTSUB_WARNING_DELETE_GROUP', $data['title']), 'warning');
			$app->redirect(JRoute::_('index.php?option=com_users&view=groups', false));
		}
		
		return;
	}
	
	// try to reset user account to a free subscription
	private static function _resetToFreeAccount($user, $profile, $oldAccount, $event = 'expired')
	{
		$free_account = SmartsubHelper::findFreeAccount();
		if($free_account)
		{
			// modify the user account in profile
			$profile->smartsub['account'] = $free_account;
			
			// we are resetting to a free account (expiration or admin block event)
			// add temp params to profile, so that we can save record to payments
			// table and send correct notification
			$profile->smartsub['account_reset'] = $event;
			$profile->smartsub['old_account'] = (int)$oldAccount;

			$user->set('smartsub', $profile->smartsub);
			$user->save(true);
			return true;
		}
		return false;
	}
	
	private static function _getAccountGroup($account_id)
	{
		$db = JFactory::getDbo();
		$db->setQuery('SELECT group_id FROM #__ss_accounts WHERE group_id = ' . (int)$account_id);
		try
		{
			return $db->loadResult();
		}
		catch(RuntimeException $e)
		{
			// log error here
			return false;
		}
	}
	
	private static function _setUserGroup($userId, $group)
	{
		$db = JFactory::getDbo();
		try
		{
			$db->setQuery('DELETE FROM #__user_usergroup_map WHERE user_id = ' . $userId);
			$db->execute();
			$db->setQuery('INSERT INTO #__user_usergroup_map (' . $db->quoteName('user_id') . ', ' . $db->quoteName('group_id') . ') VALUES (' . $userId . ', ' . $group . ')');
			$db->execute();
			
			// Apply new user permissions immediately, so that there
			// is no need to logout / login (only if the user being
			// saved is the currently logged in user)
			$session = JFactory::getSession();
			$session_user = $session->get('user');
			
			if($session_user->get('id') == $userId)
			{
				JAccess::clearStatics();
				$session_user->set('groups', array($group => $group));
				$session->set('user', $session_user);
			}
		}
		catch(RuntimeException $e)
		{
			// log error here
			return false;
		}
		return true;
	}
}
