<?php
/**
 * @package plg_user_smartsub (Smart Subscriptions User Profile Plugin) for Joomla 3
 * @version 1.1.6
 * @author Alex Polonski
 * @copyright (C)2014 - Alex Polonski
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 **/
defined('JPATH_PLATFORM') or die;

?>
<style>
	.account-tile {
		width: 50%;
		text-align: center;
	}
	@media only screen and (max-width: 480px){
		.account-tile {
			width: 80%;
			text-align: center;
		}
	}
</style>
<input type="hidden" name="<?php echo $this->name; ?>" id="<?php echo $this->id; ?>" value="<?php echo $this->value; ?>" />
		
<div><!-- Accounts info section -->
<?php
if(!$isAdmin) :
	if(COM_SMARTSUB_DEMO_MODE == 1) : 
		echo JText::_('PLG_USER_SMARTSUB_ACCOUNTS_DEMO_SPACER');
	else :
		echo JText::_('PLG_USER_SMARTSUB_ACCOUNTS_SPACER');
	endif;
endif;
?>
<?php 
foreach($accounts as $account) :
	// add unpublished note to account title (for backend we allow unpublished accounts)
	$unpublished_class = $account->published == 0 ? ' pr-unpublished' : '';
	$unpublished_notice = $account->published == 0 ? ' (' . JText::_('JUNPUBLISHED') . ')' : '';
?>
	<div id="account-info-<?php echo $account->id; ?>" class="account-info<?php echo $unpublished_class; ?>">

	<h5>
		<?php echo htmlspecialchars($account->title) . $unpublished_notice; ?> - <?php echo SmartsubHelper::formatPrice($account->full_amount, $account->currencyID, true); ?>
		<?php if(!empty($account->is_current)) : // visually mark current account ?>
			<span><i class="icon-star"></i></span>
		<?php endif; ?>
	</h5>
	<?php
		// Show account valid period information only if current account is not upgraded or 'lock_expires_on_upgrade' is OFF.
		// (SmartsubHelper::getDiscounts() will add 'expires_locked' property to accounts if expired date and time is locked)
		if(empty($account->expires_locked)) : ?>
		<?php if(!empty($account->valid_period)) : ?>
			<p><em><?php echo JText::plural('PLG_USER_SMARTSUB_N_VALID_UNIT_' . $account->valid_period_unit, $account->valid_period); ?></em></p>
		<?php else/*if($account->full_amount > 0)*/ : ?>
			<p><em><?php echo JText::_('PLG_USER_SMARTSUB_NEVER_EXPIRES'); ?></em></p>
		<?php endif; ?>
	<?php else : ?>
		<p><em><?php echo JText::sprintf('PLG_USER_SMARTSUB_EXPIRES_LOCKED', JHtml::date($account->expires_locked, JText::_('PLG_USER_SMARTSUB_DATE_TIME_FORMAT'))); ?></em></p>	
	<?php endif; ?>
	
	<?php if(!empty($account->description)) : ?>
		<div class="account-description"><?php echo JHtml::_('content.prepare', $account->description); ?></div>
	<?php endif; ?>
		<hr />
	</div>
<?php endforeach; ?>
</div>

<div><!-- Account selection section -->

<?php 
	// Add a special option for administrators, so that they can create/edit users without smartsub account
	// in their profiles - required for creating site staff members' Joomla accounts. Administrator will be
	// able to set user group(s) manually for these users
	if($isAdmin) : ?>
		<a href="javascript:void(0);" class="action-account account-tile well pull-left" data-account-id="" data-account-paid="0">
		<span><?php echo htmlspecialchars(JText::_('PLG_USER_SMARTSUB_CUSTOM_ACCOUNT')); ?></span></a>
	<?php endif; ?>

<?php
// start accounts list
foreach($accounts as $account) :
	// in backend we show unpublished accounts, add a class for unpublished accounts, so that
	// they are visually marked
	$unpublished_class = $account->published == 0 ? ' pr-unpublished' : '';
	// In account selection list we display effective amount (the actual price user will have to pay) for all accounts
	// except the current one if we are not in $account_expires_soon mode
	$display_amount = $account->is_current ? ($account_expires_soon !== false ? $account->amount : $account->full_amount) : $account->amount;
	$show_discount = !empty($account->discount) && $account->discount > 0 ? ($account->is_current ? ($account_expires_soon !== false) : true) : false;
?>
	<a href="javascript:void(0);" class="action-account account-tile well pull-left<?php echo $unpublished_class; ?>" data-account-id="<?php echo $account->id; ?>" data-account-paid="<?php echo ($account->amount > 0 ? 1 : 0); ?>" data-default-apply="<?php echo ($account->is_free ? 1 : 0); ?>" data-currency-id="<?php echo $account->currencyID; ?>" data-amount-text="<?php echo SmartsubHelper::formatFloatNumber($display_amount); ?>">
		<span><?php echo htmlspecialchars($account->title); ?></span>
		<span><?php echo htmlspecialchars(SmartsubHelper::formatPrice($display_amount, $account->currencyID)); ?></span>
	<?php if($show_discount) : ?>
		<span class="small">(<?php //echo htmlspecialchars(SmartsubHelper::formatPrice($account->full_amount, $account->currencyID)); ?>-<?php echo htmlspecialchars(SmartsubHelper::formatFloatNumber($account->discount_pct, 2)); ?>%)</span>
	<?php endif; ?>
	</a>
<?php endforeach; ?>
</div>

<?php 
	// if current account expires soon, we add the notice and control, so that the user
	// can only update profile, with no obligation of upgrading account (frontend only)
if($account_expires_soon && !$isAdmin) : ?>
	<div class="clearfix"></div>
	<div id="allow-renew-wrapper">
		<p><em><?php echo JText::sprintf('PLG_USER_SMARTSUB_RENEW_NOTICE', $account_expires_soon); ?></em></p>
		<div class="text-right">
			<button type="button" class="input-xlarge btn btn-primary action-allow-renew"><?php echo JText::_('PLG_USER_SMARTSUB_AVOID_RENEW'); ?></button>
		</div>	
	</div>
<?php endif; ?>